import fs from 'fs';
import createWebpackConfig from 'webpack5-config-react'
import { appConfigPath, resolve, webpackOverridePath } from './paths'
import { __DEV__ } from './env'
import { isSimpleObject } from '../utils';

import type { Configuration } from 'webpack'

const clientConfig: any = {}
const appConfig: any = {}
const hotMiddlewareEntry = 'webpack-hot-middleware/client'
let override = (x: Configuration) => x

const mapEntryConfig = (config: any, resolve: (data: any) => any) => {
  if (typeof config === 'string') {
    resolve(__DEV__ ? [hotMiddlewareEntry, config] : config)
  } else if (Array.isArray(config)) {
    if (__DEV__ && !config.find(x => x === hotMiddlewareEntry))
      config.unshift(hotMiddlewareEntry)
    resolve(config)
  } else if (typeof config === 'function') {
    const result = config()
    if (result instanceof Promise) result.then(v => mapEntryConfig(v, resolve))
    else mapEntryConfig(result, resolve)
  } else {
    Object.keys(config).forEach(key => {
      const value = config[key]
      if (typeof value === 'string') {
        Object.assign(config, {
          [key]: __DEV__ ? [hotMiddlewareEntry, value] : value
        })
      } else if (Array.isArray(value)) {
        if (__DEV__ && !value.find(x => x === hotMiddlewareEntry))
          value.unshift(hotMiddlewareEntry) && Object.assign(config, {
            [key]: value
          })
      }
    })
    resolve(config)
  }
}

const parseEntryConfig = (config: any) => {
  if (!config) {
    const indexPath = resolve('app/index')
    if (fs.existsSync(`${indexPath}.tsx`)) config = `${indexPath}.tsx`
    else if (fs.existsSync(`${indexPath}.jsx`)) config = `${indexPath}.jsx`
    else if (fs.existsSync(`${indexPath}.ts`)) config = `${indexPath}.ts`
    else if (fs.existsSync(`${indexPath}.js`)) config = `${indexPath}.js`
    else throw new Error('未找到入口文件, 请在 config 目录下创建 index.js 文件并导出配置, 或者在 app 目录下创建 index 文件')
  }
  return () => new Promise(resolve => mapEntryConfig(config, resolve))
}

const parseOutputConfig = (config: any) => {
  const defaultOutput = {
    path: resolve('dist'),
    publicPath: __DEV__ ? '/' : './',
    filename: __DEV__ ? '[name].js' : '[name].[contenthash:10].js',
    chunkFilename: __DEV__ ? '[name].chunk.js' : '[name].[contenthash:10].chunk.js'
  }
  if (!config) return defaultOutput
  return Object.assign(defaultOutput, config)
}

const parseClientConfig = (config: Record<string, any>) => {
  if (!isSimpleObject(config)) throw new Error('config 不是一个有效的 Object !!!')

  const { entry, output, typescript } = config

  Object.assign(clientConfig, {
    typescript,
    entry: parseEntryConfig(entry),
    output: parseOutputConfig(output)
  })
  Object.assign(appConfig, config)
}

if (fs.existsSync(appConfigPath))
  Object.assign(clientConfig, parseClientConfig(require(appConfigPath)))

if (fs.existsSync(webpackOverridePath)) override = require(webpackOverridePath)

export default createWebpackConfig(clientConfig, override)
export { appConfig }
